<?php
namespace Drupal\ot\Controller;

use Drupal\Core\Controller\ControllerBase;
use \Drupal\views\Views;
use Drupal\Core\Link;
use Drupal\Core\Url;

class OverrideMain extends ControllerBase
{

  public function getOtLocation()
  {
    return array('both'=> t('Both'), 'ui'=> t('Front/UI'), 'source'=> t('View source/HTML title tag'));
  }

  public function getOtAction()
  {
    return array(
      'ot_delete'=> t('Delete Override Title'),
      'Status'=> array(
        'ot_active'=> t('Active Override Title'),
        'ot_deactive'=> t('Deactive Override Title')
      ),
      'Location'=> array(
        'both'=> t('Both'),
        'ui'=> t('Front/UI'),
        'source'=> t('View source/HTML title tag')
      )
    );
  }

  public function checkOtAction()
  {
    return array('ot_delete', 'ot_active', 'ot_deactive', 'both', 'ui', 'source');
  }

  public function getOtById($id)
  {
    $select = db_select('override_title', 'ot')
      ->fields('ot')
      ->condition('id', $id)
      ->execute()
      ->fetchAssoc();
    return $select;
  }

  public function getTitleOfNid($nid)
  {
    $node = \Drupal\node\Entity\Node::load($nid);
    if($node){
      return $node->getTitle();
    }
  }

  public function getDisplayOfView($view_id)
  {
    $view = Views::getView(trim($view_id));
    if($view){
      $display_arr = $view->storage->get('display');
      foreach ($display_arr as $key => $value) {
        if($value['display_plugin'] == 'page'){
          $display_lists[$value['id']] = $value['display_title'];
        }
      }
    }

    if(!empty($display_lists)){
      return $display_lists;
    }
  }

  public function checkOverrideTitleByNid($nid)
  {
    $select = db_select('override_title', 'ot')
      ->fields('ot', array('id', 'title'))
      ->condition('type', 1)
      ->condition('type_id', $nid)
      ->execute()
      ->fetchAssoc();
    return $select;
  }

  public function checkOverrideTitleByView($view, $display)
  {
    $select = db_select('override_title', 'ot')
      ->fields('ot', array('id', 'title'))
      ->condition('type', 2)
      ->condition('type_id', $view)
      ->condition('display_id', $display)
      ->execute()
      ->fetchAssoc();
    return $select;
  }

  public function getOtLocationById($nid)
  {
    $select = db_select('override_title', 'ot')
      ->fields('ot', array('title', 'location'))
      ->condition('type', 1)
      ->condition('type_id', $nid)
      ->condition('status', 1)
      ->execute()
      ->fetchAssoc();
    return $select;
  }

  public function getOtLocationByView($view, $display)
  {
    $select = db_select('override_title', 'ot')
      ->fields('ot', array('title', 'location'))
      ->condition('type', 2)
      ->condition('type_id', $view)
      ->condition('display_id', $display)
      ->condition('status', 1)
      ->execute()
      ->fetchAssoc();
    return $select;
  }

  public function otDeleteById($id)
  {
    $delete = db_delete('override_title')
      ->condition('id', $id)
      ->execute();
    return $delete;
  }

  public function getOtTitle(){
    $current_path = \Drupal::request()->getRequestUri();
    $current_path_arr = explode('/', $current_path);
    $keys_arr = array_keys($current_path_arr);
    $end_key = end($keys_arr);
    $select = db_select('override_title', 'ot')
      ->fields('ot', array('title'))
      ->condition('id', $current_path_arr[$end_key-1])
      ->execute()
      ->fetchAssoc();
    return t('Edit Override Title @ottitle', ['@ottitle'=> $select['title']]);
  }

  public function getOtByIdMultiple($id_arr){
    $select = db_select('override_title', 'ot')
      ->fields('ot', array('id'))
      ->condition('id', $id_arr, 'IN')
      ->execute()
      ->fetchCol();
    return $select;
  }

  public function changeOtStatus($modify_ot_arr_val, $status){
    $update = db_update('override_title')
      ->fields([
        'status'=> $status,
        'changed'=> REQUEST_TIME,
      ])
      ->condition('id', $modify_ot_arr_val, 'IN')
      ->execute();
    return $update;
  }

  public function changeOtLocation($modify_ot_arr_val, $action_ot){
    $update = db_update('override_title')
      ->fields([
        'location'=> $action_ot,
        'changed'=> REQUEST_TIME,
      ])
      ->condition('id', $modify_ot_arr_val, 'IN')
      ->execute();
    return $update;
  }

  public function OtDeleteMultiple($modify_ot_arr_val){
    $delete = db_delete('override_title')
      ->condition('id', $modify_ot_arr_val, 'IN')
      ->execute();
    return $delete;
  }

  public function OtGetSortedData($order, $sort){
    $error_span = '<span class="error-ot"></span>';
    $query = db_select('override_title', 'n')
    ->fields('n')
    ->orderBy($order, $sort);
    $pager = $query->extend('Drupal\Core\Database\Query\PagerSelectExtender')->limit(20);
    $results = $pager->execute()->fetchAll();
    $status = ['Deactive', 'Active'];
    $type = [1=> 'Node', 'View'];
    foreach ($results as $key => $value) {
      $drop_btn = array(
        '#type' => 'dropbutton',
        '#links' => array(
          'edit' => array(
            'title' => t('Modify'),
            'url' => Url::fromRoute('override.edit', ['id'=> $value->id]),
          ),
          'delete' => array(
            'title' => t('Delete'),
            'url' => Url::fromRoute('override.delete', ['id'=> $value->id]),
          ),
        ),
      );

      if($value->type == 1){
        if($this->getTitleOfNid($value->type_id)){
          $url = Url::fromUserInput('/node/'.$value->type_id);
          $anchor_type_id = Link::fromTextAndUrl(t('node/'.$value->type_id), $url)->toString();
        }
        else{
          $anchor_type_id = $error_span.'node/'.$value->type_id.' not found.';
        }
      }
      else if($value->type == 2){
        $view = Views::getView(trim($value->type_id));
        if($view){
          $url = Url::fromRoute('view.'.$value->type_id.'.'.$value->display_id);
          $anchor_type_id = $view->storage->get('label').' ('.$value->type_id.')';

          if(array_key_exists($value->display_id, $this->getDisplayOfView($value->type_id))){
            $anchor_display_id = Link::fromTextAndUrl(t($view->storage->get('display')[$value->display_id]['display_title'].' ('.$value->display_id.')'), $url)->toString();
          }
          else{
            $anchor_display_id = $error_span.$value->display_id.': not found.';
          }
        }
        else{
          $anchor_type_id = $error_span.$value->type_id.': view not exists';
          $anchor_display_id = !empty($value->display_id) ? $value->display_id : 'empty';
          $anchor_display_id = $error_span.$anchor_display_id;
        }
      }
      else{
        $anchor_type_id = $error_span.$value->type_id;
        $anchor_display_id = $error_span.'NULL';
      }


      $options[$value->id] = [
        (in_array($value->type, [1, 2])) ? $type[$value->type] : $error_span.'Unknown',
        $anchor_type_id,
        !empty($value->type == 2) ? $anchor_display_id : '',
        $value->title,
        (in_array($value->location, ['both', 'ui', 'source'])) ? $this->getOtLocation()[$value->location] : $error_span.'Unknown',
        (in_array($value->status, [0, 1])) ? $status[$value->status] : $error_span.'Unknown',
        format_date($value->changed, 'custom', 'd/m/Y - H:m'),
        format_date($value->created, 'custom', 'd/m/Y - H:m'),
        array('data'=> $drop_btn)
      ];
    }

    $header = array(
      ['data' => t('type'), 'field' => 't.type'],
      'ID(Node/View)',
      'Display ID',
      ['data' => t('title'), 'field' => 't.title'],
      'Location',
      ['data' => t('status'), 'field' => 't.status'],
      'Modified',
      ['data' => t('created'), 'field' => 't.created', 'sort' => 'desc'],
      'Operations'
    );

    return array($header, @$options);
  }

}
