<?php
namespace Drupal\ot\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\ot\Controller\OverrideMain;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

class Edit extends FormBase
{
  public function __construct() {
    $this->ot_main = new OverrideMain();
    $current_path = \Drupal::request()->getRequestUri();
    $current_path_arr = explode('/', $current_path);
    $keys_arr = array_keys($current_path_arr);
    $end_key = end($keys_arr);
    $this->id = $current_path_arr[$end_key-1];
    $this->default = $this->ot_main->getOtById($this->id);
    $display_return = $this->ot_main->getDisplayOfView($this->default['type_id']);
    if($display_return){
      $this->options = array('' => '- Select -')+$display_return;
    }
  }

  public function getFormId()
  {
    return 'override_title_edit';
  }

  public function buildForm(array $form, FormStateInterface $form_state)
  {
    if(!$this->ot_main->getOtById($this->id)){
      throw new NotFoundHttpException();
    }
    $form['#attached']['library'][] = 'ot/ot.lib';
    $form['ot_type'] = array(
      '#type'=> 'radios',
      '#title'=> t('Type of page'),
      '#options'=> array(1=> 'Node', 'View'),
      '#default_value'=> $this->default['type'],
      '#required'=> true,
    );

    $form['ot_type_id'] = array(
      '#type'=> 'textfield',
      '#title'=> t('Type ID(node_id/machine_name)'),
      '#default_value'=> $this->default['type_id'],
      '#required'=> true,
      '#description' => t("Enter machine_name or node_id in the above text box. Eg: node_id as 125 not as 'node/125'."),
      '#attributes'=> array(
        'placeholder'=> t('Enter node_id or machine_name.')
      ),
      '#ajax'=> array(
        'callback'=> '::otTypeExists',
        'effect' => 'fade',
        'event'=> 'change',
        'progress' => array(
          'type' => 'throbber',
          'message' => t('Collecting information...'),
        ),
      )
    );

    $ot_title = !empty($this->options) ? $this->default['type_id'] : '';
    $form['ot_display'] = array(
      '#type'=> 'select',
      '#title'=> t('View dispaly of: '.$ot_title),
      '#options'=> @$this->options,
      '#default_value'=> !empty($ot_title)? $this->default['display_id'] :'',
      '#empty_options'=> t('- Select display -'),
      '#required'=> (($form_state->getValue('ot_type') == 2) || !empty($this->default['display_id'])) ? true : false,
      '#validated' => true,
      '#states'=> array(
        'visible'=> array(
          'input[name="ot_type"]'=> array('value'=> 2),
          'input[name="ot_type_id"]'=> array('filled' => TRUE),
        ),
        'disabled'=> array(
          'input[name="ot_type"]'=> array('value'=> 1)
        )
      ),
      '#prefix'=> '<div id="ot-type-id-exists">',
      '#suffix'=> '</div>',
    );

    $form['ot_title'] = array(
      '#type'=> 'textfield',
      '#title'=> t('Override Title'),
      '#default_value'=> $this->default['title'],
      '#required'=> true,
      '#attributes'=> array(
        'placeholder'=> t('Enter title to Override.')
      )
    );

    $form['ot_location'] = array(
      '#type'=> 'radios',
      '#title'=> t('OT action place/location'),
      '#options'=> $this->ot_main->getOtLocation(),
      '#default_value'=> $this->default['location'],
      '#required'=> true
    );

    $form['ot_status'] = array(
      '#type'=> 'checkbox',
      '#title'=> t('Active'),
      '#attributes'=> array(
        'checked'=> ($this->default['status'] == 1) ? true : false
      )
    );

    $form['actions'] = array(
      '#type' => 'actions'
    );

    $form['actions']['ot_submit'] = array(
      '#type'=> 'submit',
      '#value'=> t('Submit'),
      '#attributes'=> array(
        'class'=> array('button button--primary')
      )
    );

    $form['actions']['ot_delete'] = array(
      '#type'=> 'link',
      '#title'=> t('Delete'),
      '#url' => Url::fromRoute('override.delete', ['id'=> $this->id]),
      '#attributes'=> array(
        'class'=> array('button button--danger')
      )
    );

    return $form;
  }

  public function otTypeExists(array &$form, FormStateInterface $form_state)
  {
    $ot_type = $form_state->getValue('ot_type');
    $ot_type_id = $form_state->getValue('ot_type_id');
    if($ot_type == 1){
      if(!is_numeric($ot_type_id)){
        $elm = '<div role="contentinfo" aria-label="Error message" class="messages messages--error"><div role="alert"><h2 class="visually-hidden">Error message</h2> Please enter numeric value as node_id.</div></div>';
      }
      else{
        $node_title_return = $this->ot_main->getTitleOfNid($ot_type_id);
        if(!empty($node_title_return)){
          $elm = array(
            '#type'=> 'textfield',
            '#title'=> t('Title of Nid: '. $ot_type_id),
            '#value'=> $node_title_return,
            '#description' => t("Shows title of the above 'Node Id' field, If exists."),
            '#attributes'=> array(
              'placeholder'=> t('Node title.'),
              'readonly'=>true
            ),
          );
        }else{
          $elm = '<div role="contentinfo" aria-label="Error message" class="messages messages--error"><div role="alert"><h2 class="visually-hidden">Error message</h2> Node Id: '.$ot_type_id.' does not exist.</div></div>';
        }
      }
    }else if($ot_type == 2){
      $display_return = $this->ot_main->getDisplayOfView($ot_type_id);
      if(!empty($display_return)){
        $display_return = array('' => '- Select -')+$display_return;
        $form['ot_display']['#title'] = 'View dispaly of: '.$ot_type_id;
        $form['ot_display']['#options'] = $display_return;
        $form['ot_display']['#value'] = '';
        $form['ot_display']['#required'] = true;
        $form['ot_display']['#prefix'] = '';
        $form['ot_display']['#suffix'] = '';
        $elm = drupal_render($form['ot_display']);
      }else{
        $elm = '<div role="contentinfo" aria-label="Error message" class="messages messages--error"><div role="alert"><h2 class="visually-hidden">Error message</h2> View machine_name "'.$ot_type_id.'" does not exist or not have any dispaly page.</div></div>';
      }
    }else{
      $elm = '<div role="contentinfo" aria-label="Error message" class="messages messages--error"><div role="alert"><h2 class="visually-hidden">Error message</h2> "Type of page" field value has been interrupted. Please try again.</div></div>';
    }

    $ajaxresponse = new AjaxResponse();
    $ajaxresponse->addCommand(new HtmlCommand('#ot-type-id-exists', $elm));
    return $ajaxresponse;
  }

  public function validateForm(array &$form, FormStateInterface $form_state)
  {
    $ot_type = $form_state->getValue('ot_type');
    $ot_type_id = $form_state->getValue('ot_type_id');
    $ot_display = @$form_state->getValue('ot_display');
    $ot_location = $form_state->getValue('ot_location');
    if(!in_array($ot_type, [1,2])){
      $form_state->setErrorByName('ot_type', t("'Type of page' - An illegal choice has been detected. Please contact the site administrator."));
    }

    if(empty($ot_type_id)){
      $form_state->setErrorByName('ot_type_id', t('Type ID cannot be empty.'));
    }

    if($ot_type == 1){
      if(!is_numeric($ot_type_id)){
        $form_state->setErrorByName('ot_type_id', t('Type ID entered should be numeric for Node.'));
      }
      else{
        $node_title_return = $this->ot_main->getTitleOfNid($ot_type_id);
        if(empty($node_title_return)){
          $form_state->setErrorByName('ot_type_id', t('Type ID entered as node_id entered does not exist.'));
        }
        else{
          $ot_title_return = $this->ot_main->checkOverrideTitleByNid($ot_type_id);
          if(!empty($ot_title_return['id']) && $ot_title_return['id']!= $this->id){
            $form_state->setErrorByName('ot_type_id', t('node_id: '.$ot_type_id. ', title has already been override.'));
          }
        }
      }
    }
    else if($ot_type == 2){
      $display_return = $this->ot_main->getDisplayOfView($ot_type_id);
      if(empty($display_return)){
        $form_state->setErrorByName('ot_type_id', t('Type ID as view does not exists or not have any display page.'));
      }

      if(empty($ot_display)){
        $form_state->setErrorByName('ot_display', t('View dispaly cannot be empty.'));
      }

      if(!empty($display_return) && !empty($ot_display)){
        $ot_title_return = $this->ot_main->checkOverrideTitleByView($ot_type_id, $ot_display);
        if(!empty($ot_title_return['id']) &&  $ot_title_return['id'] != $this->id){
          $form_state->setErrorByName('ot_type_id', t("View 'machine_name': ".$ot_type_id. " and 'display': ".$ot_display. ", title has already been override."));
          $form_state->setErrorByName('ot_display', t("This view 'display': ".$ot_display. ", already in use."));
        }
      }
      $form['ot_display']['#title'] = 'View dispaly of: '.$ot_type_id;
      if(!empty($display_return)){
        $display_return = array('' => '- Select -')+$display_return;
        $form['ot_display']['#options'] = $display_return;
        $form['ot_display']['#default_value'] = $ot_display;
      }
    }

    if(empty($ot_location)){
      $form_state->setErrorByName('ot_location', t("'OT action place/location' cannot be empty."));
    }

    if(!in_array($ot_location, ['both', 'ui', 'source'])){
      $form_state->setErrorByName('ot_location', t("''OT action place/location' - An illegal choice has been detected. Please contact the site administrator."));
    }
  }

  public function submitForm(array &$form, FormStateInterface $form_state)
  {
    $display_id = ($form_state->getValue('ot_type') == 2) ? $form_state->getValue('ot_display') : null;
    try{
      db_update('override_title')
      ->fields([
        'type'=> $form_state->getValue('ot_type'),
        'type_id'=> $form_state->getValue('ot_type_id'),
        'display_id'=> $display_id,
        'title' => $form_state->getValue('ot_title'),
        'location' => $form_state->getValue('ot_location'),
        'status' => $form_state->getValue('ot_status'),
        'changed'=> REQUEST_TIME,
      ])
      ->condition('id', $this->id)
      ->execute();
      drupal_set_message(t("ID(node_id/machine_name): '".$form_state->getValue('ot_type_id')."' title has been override."), 'status');
      $url = Url::fromRoute('override.ot');
      $form_state->setRedirectUrl($url);
    }
    catch (Exception $e){
      drupal_set_message(t("ID(node_id/machine_name): '".$form_state->getValue('ot_type_id')."' title cannot be override. Please try again!"), 'error');
    }
  }
}
